@@ -33,6 +33,13 @@ class Miniscript(DescriptorBase):
         ]
         return type(self)(*args)
 
+    def branch(self, branch_index):
+        args = [
+            arg.branch(branch_index) if hasattr(arg, "branch") else arg
+            for arg in self.args
+        ]
+        return type(self)(*args)
+
     @property
     def properties(self):
         return self.PROPS
@@ -578,11 +585,11 @@ class Thresh(Miniscript):
         return self.len_args() + len(self.args) - 1
 
     def verify(self):
-        # 1 < k < n; X1 is Bdu; others are Wdu
+        # 1 <= k <= n; X1 is Bdu; others are Wdu
         super().verify()
-        if self.args[0].num <= 1 or self.args[0].num >= (len(self.args) - 1):
+        if self.args[0].num < 1 or self.args[0].num >= len(self.args):
             raise MiniscriptError(
-                "thresh: Invalid k! Should be 1 < k < %d, got %d"
+                "thresh: Invalid k! Should be 1 <= k <= %d, got %d"
                 % (len(self.args) - 1, self.args[0].num)
             )
         if self.args[1].type != "B":
