@@ -159,46 +159,111 @@ def _init(flags=(CONTEXT_SIGN | CONTEXT_VERIFY)):
     secp256k1.secp256k1_ec_pubkey_combine.restype = c_int
 
     # recoverable module
-    secp256k1.secp256k1_ecdsa_sign_recoverable.argtypes = [
-        c_void_p,
-        c_char_p,
-        c_char_p,
-        c_char_p,
-        c_void_p,
-        c_void_p,
-    ]
-    secp256k1.secp256k1_ecdsa_sign_recoverable.restype = c_int
+    try:
+        secp256k1.secp256k1_ecdsa_sign_recoverable.argtypes = [
+            c_void_p,
+            c_char_p,
+            c_char_p,
+            c_char_p,
+            c_void_p,
+            c_void_p,
+        ]
+        secp256k1.secp256k1_ecdsa_sign_recoverable.restype = c_int
 
-    secp256k1.secp256k1_ecdsa_recoverable_signature_parse_compact.argtypes = [
-        c_void_p,
-        c_char_p,
-        c_char_p,
-        c_int,
-    ]
-    secp256k1.secp256k1_ecdsa_recoverable_signature_parse_compact.restype = c_int
+        secp256k1.secp256k1_ecdsa_recoverable_signature_parse_compact.argtypes = [
+            c_void_p,
+            c_char_p,
+            c_char_p,
+            c_int,
+        ]
+        secp256k1.secp256k1_ecdsa_recoverable_signature_parse_compact.restype = c_int
 
-    secp256k1.secp256k1_ecdsa_recoverable_signature_serialize_compact.argtypes = [
-        c_void_p,
-        c_char_p,
-        c_char_p,
-        c_char_p,
-    ]
-    secp256k1.secp256k1_ecdsa_recoverable_signature_serialize_compact.restype = c_int
+        secp256k1.secp256k1_ecdsa_recoverable_signature_serialize_compact.argtypes = [
+            c_void_p,
+            c_char_p,
+            c_char_p,
+            c_char_p,
+        ]
+        secp256k1.secp256k1_ecdsa_recoverable_signature_serialize_compact.restype = c_int
 
-    secp256k1.secp256k1_ecdsa_recoverable_signature_convert.argtypes = [
-        c_void_p,
-        c_char_p,
-        c_char_p,
-    ]
-    secp256k1.secp256k1_ecdsa_recoverable_signature_convert.restype = c_int
+        secp256k1.secp256k1_ecdsa_recoverable_signature_convert.argtypes = [
+            c_void_p,
+            c_char_p,
+            c_char_p,
+        ]
+        secp256k1.secp256k1_ecdsa_recoverable_signature_convert.restype = c_int
 
-    secp256k1.secp256k1_ecdsa_recover.argtypes = [
-        c_void_p,
-        c_char_p,
-        c_char_p,
-        c_char_p,
-    ]
-    secp256k1.secp256k1_ecdsa_recover.restype = c_int
+        secp256k1.secp256k1_ecdsa_recover.argtypes = [
+            c_void_p,
+            c_char_p,
+            c_char_p,
+            c_char_p,
+        ]
+        secp256k1.secp256k1_ecdsa_recover.restype = c_int
+    except:
+        pass
+
+    # zkp modules
+    try:
+        # generator module
+        secp256k1.secp256k1_generator_parse.argtypes = [c_void_p, c_char_p, c_char_p]
+        secp256k1.secp256k1_generator_parse.restype = c_int
+
+        secp256k1.secp256k1_generator_serialize.argtypes = [c_void_p, c_char_p, c_char_p]
+        secp256k1.secp256k1_generator_serialize.restype = c_int
+
+        secp256k1.secp256k1_generator_generate.argtypes = [c_void_p, c_char_p, c_char_p]
+        secp256k1.secp256k1_generator_generate.restype = c_int
+
+        secp256k1.secp256k1_generator_generate_blinded.argtypes = [c_void_p, c_char_p, c_char_p, c_char_p]
+        secp256k1.secp256k1_generator_generate_blinded.restype = c_int
+        
+        # pederson commitments
+        secp256k1.secp256k1_pedersen_commitment_parse.argtypes = [c_void_p, c_char_p, c_char_p]
+        secp256k1.secp256k1_pedersen_commitment_parse.restype = c_int
+
+        secp256k1.secp256k1_pedersen_commitment_serialize.argtypes = [c_void_p, c_char_p, c_char_p]
+        secp256k1.secp256k1_pedersen_commitment_serialize.restype = c_int
+
+        secp256k1.secp256k1_pedersen_commit.argtypes = [c_void_p, c_char_p, c_char_p, c_uint64, c_char_p]
+        secp256k1.secp256k1_pedersen_commit.restype = c_int
+
+        # rangeproof
+        secp256k1.secp256k1_rangeproof_rewind.argtypes = [c_void_p, c_char_p, POINTER(c_uint64), c_char_p, POINTER(c_uint64),
+                                                          c_char_p, POINTER(c_uint64), POINTER(c_uint64),
+                                                          c_char_p, c_char_p, c_uint64,
+                                                          c_char_p, c_uint64,
+                                                          c_char_p]
+        secp256k1.secp256k1_rangeproof_rewind.restype = c_int
+
+        secp256k1.secp256k1_rangeproof_sign.argtypes = [
+          c_void_p, # ctx
+          c_char_p, # proof
+          POINTER(c_uint64), # plen
+          c_uint64, # min_value
+          c_char_p, # commit
+          c_char_p, # blind
+          c_char_p, # nonce
+          c_int,    # exp
+          c_int,    # min_bits
+          c_uint64, # value
+          c_char_p, # message
+          c_uint64, # msg_len
+          c_char_p, # extra_commit
+          c_uint64, # extra_commit_len
+          c_char_p, # gen
+        ]
+        secp256k1.secp256k1_rangeproof_sign.restype = c_int
+
+        # musig
+
+        secp256k1.secp256k1_xonly_pubkey_from_pubkey.argtypes = [c_void_p, c_char_p, POINTER(c_int), c_char_p]
+        secp256k1.secp256k1_xonly_pubkey_from_pubkey.restype = c_int
+
+        secp256k1.secp256k1_musig_pubkey_combine.argtypes = [c_void_p, c_void_p, c_char_p, c_void_p, c_void_p, c_size_t]
+        secp256k1.secp256k1_musig_pubkey_combine.restype = c_int
+    except:
+        pass
 
     secp256k1.ctx = secp256k1.secp256k1_context_create(flags)
 
@@ -465,3 +530,138 @@ def ecdsa_recover(sig, msghash, context=_secp.ctx):
     if r == 0:
         raise ValueError("Failed to recover public key")
     return pub
+
+# zkp modules
+
+def pedersen_commitment_parse(inp, context=_secp.ctx):
+    if len(inp)!=33:
+        raise ValueError("Serialized commitment should be 33 bytes long")
+    commit = bytes(64)
+    r = _secp.secp256k1_pedersen_commitment_parse(context, commit, inp)
+    if r == 0:
+        raise ValueError("Failed to parse commitment")
+    return commit
+
+def pedersen_commitment_serialize(commit, context=_secp.ctx):
+    if len(commit)!=64:
+        raise ValueError("Commitment should be 64 bytes long")
+    sec = bytes(33)
+    r = _secp.secp256k1_pedersen_commitment_serialize(context, sec, commit)
+    if r == 0:
+        raise ValueError("Failed to serialize commitment")
+    return sec
+
+def pedersen_commit(vbf, value, gen, context=_secp.ctx):
+    if len(gen)!=64:
+        raise ValueError("Generator should be 64 bytes long")
+    if len(vbf)!=32:
+        raise ValueError("Blinding factor should be 32 bytes long")
+    commit = bytes(64)
+    r = _secp.secp256k1_pedersen_commit(context, commit, vbf, value, gen)
+    if r == 0:
+        raise ValueError("Failed to create commitment")
+    return commit
+
+# generator
+def generator_parse(inp, context=_secp.ctx):
+    if len(inp)!=33:
+        raise ValueError("Serialized generator should be 33 bytes long")
+    gen = bytes(64)
+    r = _secp.secp256k1_generator_parse(context, gen, inp)
+    if r == 0:
+        raise ValueError("Failed to parse generator")
+    return gen
+
+def generator_generate(asset, context=_secp.ctx):
+    if len(asset)!=32:
+        raise ValueError("Asset should be 32 bytes long")
+    gen = bytes(64)
+    r = _secp.secp256k1_generator_generate(context, gen, asset)
+    if r == 0:
+        raise ValueError("Failed to generate generator")
+    return gen
+
+def generator_generate_blinded(asset, abf, context=_secp.ctx):
+    if len(asset)!=32:
+        raise ValueError("Asset should be 32 bytes long")
+    if len(abf)!=32:
+        raise ValueError("Asset blinding factor should be 32 bytes long")
+    gen = bytes(64)
+    r = _secp.secp256k1_generator_generate_blinded(context, gen, asset, abf)
+    if r == 0:
+        raise ValueError("Failed to generate generator")
+    return gen
+
+def generator_serialize(generator, context=_secp.ctx):
+    if len(generator)!=64:
+        raise ValueError("Generator should be 64 bytes long")
+    sec = bytes(33)
+    if _secp.secp256k1_generator_serialize(context, sec, generator) == 0:
+        raise RuntimeError("Failed to serialize generator")
+    return sec
+
+# rangeproof
+def rangeproof_rewind(proof, nonce, value_commitment, script_pubkey, generator, message_length=64, context=_secp.ctx):
+    if len(generator)!=64:
+        raise ValueError("Generator should be 64 bytes long")
+    if len(nonce)!=32:
+        raise ValueError("Nonce should be 32 bytes long")
+    if len(value_commitment)!=64:
+        raise ValueError("Value commitment should be 64 bytes long")
+
+    msg = b"\x00"*message_length
+    pointer = POINTER(c_uint64)
+    msglen = pointer(c_uint64(len(msg)))
+
+    vbf_out = b"\x00"*32
+    value_out = pointer(c_uint64(0))
+    min_value = pointer(c_uint64(0))
+    max_value = pointer(c_uint64(0))
+    res = _secp.secp256k1_rangeproof_rewind(context, vbf_out, value_out,
+                            msg, msglen,
+                            nonce, min_value, max_value,
+                            value_commitment, proof, len(proof),
+                            script_pubkey, len(script_pubkey),
+                            generator)
+    if res != 1:
+        raise RuntimeError("Failed to rewind the proof")
+    return value_out.contents.value, vbf_out, msg[:msglen.contents.value], min_value.contents.value, max_value.contents.value
+
+def rangeproof_sign(nonce, value, value_commitment, vbf, message, extra, gen, min_value=1, exp=0, min_bits=52, context=_secp.ctx):
+    if len(gen)!=64:
+        raise ValueError("Generator should be 64 bytes long")
+    if len(nonce)!=32:
+        raise ValueError("Nonce should be 32 bytes long")
+    if len(value_commitment)!=64:
+        raise ValueError("Value commitment should be 64 bytes long")
+    if len(vbf)!=32:
+        raise ValueError("Value blinding factor should be 32 bytes long")
+    proof = bytes(5134)
+    pointer = POINTER(c_uint64)
+    prooflen = pointer(c_uint64(len(proof)))
+    res = _secp.secp256k1_rangeproof_sign(context, proof, prooflen,
+                min_value, value_commitment, vbf, nonce,
+                exp, min_bits, value, message, len(message), extra, len(extra), gen)
+    if res != 1:
+        raise RuntimeError("Failed to generate the proof")
+    return bytes(proof[:prooflen.contents.value])
+
+def xonly_pubkey_from_pubkey(pubkey, context=_secp.ctx):
+    if len(pubkey)!=64:
+        raise ValueError("Pubkey should be 64 bytes long")
+    pointer = POINTER(c_int)
+    parity = pointer(c_int(0))
+    xonly_pub = bytes(64)
+    res = _secp.secp256k1_xonly_pubkey_from_pubkey(context, xonly_pub, parity, pubkey)
+    if res != 1:
+        raise RuntimeError("Failed to convert the pubkey")
+    return xonly_pub, bool(parity.contents.value)
+
+def musig_pubkey_combine(*args, context=_secp.ctx):
+    pub = bytes(64)
+    # TODO: strange that behaviour is different from pubkey_combine...
+    pubkeys = b"".join(args) # (c_char_p * len(args))(*args)
+    res = _secp.secp256k1_musig_pubkey_combine(context, None, pub, None, pubkeys, len(args))
+    if res == 0:
+        raise ValueError("Failed to combine pubkeys")
+    return pub
